#include "buzzer_mlp101c.h"

static uint32_t buzzer_state = 0;
static uint32_t buzzer_on = 0;
static uint32_t buzzer_ontime; // buzzer on duration
static uint32_t buzzer_offtime; // buzzer off duration
static uint32_t buzzer_repeat; // repeat count
static uint32_t buzzer_counter;

void buzzer_init(void)
{
  NVIC_InitTypeDef NVIC_InitStructure;
  GPIO_InitTypeDef GPIO_InitStructure;
  TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;

  /* TIM2 clock enable */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);

  /* GPIOC clock enable */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);
  
  /* Enable the TIM2 global Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;

  NVIC_Init(&NVIC_InitStructure);
  
  /* GPIOC Configuration:Pin14 push-pull */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_14;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  GPIO_Init(GPIOC, &GPIO_InitStructure);
  
  /* Time base configuration */
  TIM_TimeBaseStructure.TIM_Period = 9000 - 1;
  TIM_TimeBaseStructure.TIM_Prescaler = 0;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

  TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

  /* TIM IT enable */
  TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);
}

void TIM2_IRQHandler(void)
{
    TIM_ClearITPendingBit(TIM2, TIM_IT_Update);

    switch(buzzer_state)
    {
      case 0:
        break;
      case 1: //buzzer on
        buzzer_counter++;
        if(buzzer_counter < buzzer_ontime)
        {
          buzzer_on = 1;
        }
        else
        {
          buzzer_on = 0;
          buzzer_repeat--;
          if(buzzer_repeat != 0)
          {
            buzzer_state = 2;
            buzzer_counter = 0;
          }
          else
          {
            TIM_Cmd(TIM2, DISABLE);
            buzzer_state = 0;
          }
        }

        break;
      case 2: //buzzer off state before next on
        buzzer_counter++;
        if(buzzer_counter == buzzer_offtime)
        {
          buzzer_counter = 0;
          buzzer_state = 1;
        }
        break;
      default:
        buzzer_on = 0;
        TIM_Cmd(TIM2, DISABLE);
        break;
    }

  /* Pin PC.14 toggling with frequency = 4kHz */
  if(buzzer_on)
    GPIO_WriteBit(GPIOC, GPIO_Pin_14, (BitAction)(1 - GPIO_ReadOutputDataBit(GPIOC, GPIO_Pin_14)));
}

void buzzer(uint32_t type, uint32_t repeat)
{
  if(buzzer_state == 0)
  {
    switch(type)
    {
      case 0: // short beep
        buzzer_ontime = 800;
        buzzer_offtime = 500;
        buzzer_counter = 0;
        buzzer_repeat = repeat;
        buzzer_state = 1;
        TIM_Cmd(TIM2, ENABLE);
        break;
      case 1: // long beep
        buzzer_ontime = 5000;
        buzzer_offtime = 2000;
        buzzer_counter = 0;
        buzzer_repeat = repeat;
        buzzer_state = 1;
        TIM_Cmd(TIM2, ENABLE);
        break;
      default:
        TIM_Cmd(TIM2, DISABLE);
        break;
    }
  }
}
