#include "stm32f10x.h"
#include "eeprom_mlp101c.h"

#define EEPROMCS_PIN          GPIO_Pin_3
#define EEPROMCS_GPIO_PORT         GPIOC
#define EEPROMCS_GPIO_CLK     RCC_APB2Periph_GPIOC

#define EEPROMSK_PIN          GPIO_Pin_2
#define EEPROMSK_GPIO_PORT         GPIOC
#define EEPROMSK_GPIO_CLK     RCC_APB2Periph_GPIOC

#define EEPROMDI_PIN          GPIO_Pin_1
#define EEPROMDI_GPIO_PORT         GPIOC
#define EEPROMDI_GPIO_CLK     RCC_APB2Periph_GPIOC

#define EEPROMDO_PIN          GPIO_Pin_0
#define EEPROMDO_GPIO_PORT         GPIOC
#define EEPROMDO_GPIO_CLK     RCC_APB2Periph_GPIOC

#define EEPROMCSL GPIO_WriteBit(EEPROMCS_GPIO_PORT, EEPROMCS_PIN, Bit_RESET)
#define EEPROMCSH GPIO_WriteBit(EEPROMCS_GPIO_PORT, EEPROMCS_PIN, Bit_SET)
#define EEPROMSKL GPIO_WriteBit(EEPROMSK_GPIO_PORT, EEPROMSK_PIN, Bit_RESET)
#define EEPROMSKH GPIO_WriteBit(EEPROMSK_GPIO_PORT, EEPROMSK_PIN, Bit_SET)
#define EEPROMDIL GPIO_WriteBit(EEPROMDI_GPIO_PORT, EEPROMDI_PIN, Bit_RESET)
#define EEPROMDIH GPIO_WriteBit(EEPROMDI_GPIO_PORT, EEPROMDI_PIN, Bit_SET)


void eeprom_init(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  /* Enable GPIO clock */
  RCC_APB2PeriphClockCmd(EEPROMCS_GPIO_CLK | EEPROMSK_GPIO_CLK | EEPROMDI_GPIO_CLK | EEPROMDO_GPIO_CLK, ENABLE);

  /* Configure EEPROM_CS in Output Push-Pull mode */
  GPIO_InitStructure.GPIO_Pin = EEPROMCS_PIN;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_Init(EEPROMCS_GPIO_PORT, &GPIO_InitStructure);

  // EEPROMSK output
  GPIO_InitStructure.GPIO_Pin = EEPROMSK_PIN;
  GPIO_Init(EEPROMSK_GPIO_PORT, &GPIO_InitStructure);
  
  // EEPROMDI output
  GPIO_InitStructure.GPIO_Pin = EEPROMDI_PIN;
  GPIO_Init(EEPROMDI_GPIO_PORT, &GPIO_InitStructure);
  
  // EEPROMDO input
  GPIO_InitStructure.GPIO_Pin = EEPROMDO_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_Init(EEPROMDO_GPIO_PORT, &GPIO_InitStructure);

  eeprom_ewen();
}

uint16_t eeprom_read(uint8_t add)
{
  uint16_t i;
  uint32_t didata;
  uint32_t dodata = 0;
  didata = 0xc0000000 | (add << 23);
  //printf("di:%x\r\n", didata);
  EEPROMCSL;
  EEPROMSKL;
  EEPROMDIL;
  
  __disable_interrupt(); // for minimum SK constraint
  EEPROMCSH;
  __no_operation(); // for tcss 50ns
  __no_operation();
  __no_operation();
  for(i = 0; i < 25; i++)
  {
    if(0x80000000 & didata)
      EEPROMDIH;
    else
      EEPROMDIL;
    __no_operation(); // for tskl 250ns
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    EEPROMSKH;
    __no_operation(); // for tskh 250ns
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    EEPROMSKL;
    didata <<= 1;
    dodata <<= 1;
    if(GPIO_ReadInputDataBit(EEPROMDO_GPIO_PORT, EEPROMDO_PIN))
      dodata |= 1;
  }
  EEPROMCSL;
  __enable_interrupt();
  __no_operation(); // for tcs 250ns
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  return (uint16_t)(dodata & 0x0000ffff);
}

void eeprom_ewen(void)
{
  uint16_t i;
  uint32_t didata;
  didata = 0x98000000;
  
  EEPROMCSL;
  EEPROMSKL;
  EEPROMDIL;
  
  __disable_interrupt(); // for minimum SK constraint
  EEPROMCSH;
  __no_operation(); // for tcss 50ns
  __no_operation();
  __no_operation();
  for(i = 0; i < 8; i++)
  {
    if(0x80000000 & didata)
      EEPROMDIH;
    else
      EEPROMDIL;
    __no_operation(); // for tskl 250ns
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    EEPROMSKH;
    __no_operation(); // for tskh 250ns
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    EEPROMSKL;
    didata <<= 1;

  }
  EEPROMCSL;
  __enable_interrupt();
  __no_operation(); // for tcs 250ns
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
}

void eeprom_ewds(void)
{
  uint16_t i;
  uint32_t didata;
  didata = 0x90000000;
  
  EEPROMCSL;
  EEPROMSKL;
  EEPROMDIL;
  
  __disable_interrupt(); // for minimum SK constraint
  EEPROMCSH;
  __no_operation(); // for tcss 50ns
  __no_operation();
  __no_operation();
  for(i = 0; i < 8; i++)
  {
    if(0x80000000 & didata)
      EEPROMDIH;
    else
      EEPROMDIL;
    __no_operation(); // for tskl 250ns
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    EEPROMSKH;
    __no_operation(); // for tskh 250ns
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    EEPROMSKL;
    didata <<= 1;

  }
  EEPROMCSL;
  __enable_interrupt();
  __no_operation(); // for tcs 250ns
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
}

uint16_t eeprom_write(uint8_t add, uint16_t data)
{
  uint16_t i;
  uint32_t didata;
  didata = 0xa0000000 | (add << 23) | (data << 7);
  //printf("di:%x\r\n", didata);
  EEPROMCSL;
  EEPROMSKL;
  EEPROMDIL;
  
  __disable_interrupt(); // for minimum SK constraint
  EEPROMCSH;
  __no_operation(); // for tcss 50ns
  __no_operation();
  __no_operation();
  for(i = 0; i < 25; i++)
  {
    if(0x80000000 & didata)
      EEPROMDIH;
    else
      EEPROMDIL;
    __no_operation(); // for tskl 250ns
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    EEPROMSKH;
    __no_operation(); // for tskh 250ns
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    __no_operation();
    EEPROMSKL;
    didata <<= 1;
  }
  EEPROMCSL;
  __enable_interrupt();
  
  __no_operation(); // for tcs 250ns
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  EEPROMCSH;
  __no_operation();
  __no_operation();
  while(!GPIO_ReadInputDataBit(EEPROMDO_GPIO_PORT, EEPROMDO_PIN));
  EEPROMCSL;
  __no_operation(); // for tcs 250ns
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
  __no_operation();
}
