#include "stm32f10x.h"
#include "lcd_mlp101c.h"
#include "utils.h"


#define LCD_E_PIN               GPIO_Pin_12                  
#define LCD_E_GPIO_PORT         GPIOC                       
#define LCD_E_GPIO_CLK          RCC_APB2Periph_GPIOC  
#define LCD_RW_PIN              GPIO_Pin_11
#define LCD_RW_GPIO_PORT        GPIOC
#define LCD_RW_GPIO_CLK         RCC_APB2Periph_GPIOC
#define LCD_RS_PIN              GPIO_Pin_10                     
#define LCD_RS_GPIO_PORT        GPIOC
#define LCD_RS_GPIO_CLK         RCC_APB2Periph_GPIOC

#define LCD_D0_PIN              GPIO_Pin_0
#define LCD_D1_PIN              GPIO_Pin_1
#define LCD_D2_PIN              GPIO_Pin_2
#define LCD_D3_PIN              GPIO_Pin_3
#define LCD_D4_PIN              GPIO_Pin_4
#define LCD_D5_PIN              GPIO_Pin_5
#define LCD_D6_PIN              GPIO_Pin_6
#define LCD_D7_PIN              GPIO_Pin_7
#define LCD_D_GPIO_PORT         GPIOD
#define LCD_D_GPIO_CLK          RCC_APB2Periph_GPIOD

static void delay(vu32 nCount);

static void LCD_CtrlLinesConfig(void);
static void LCD_write(uint8_t data, uint8_t rs);

static void LCD_CtrlLinesConfig(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  /* Enable GPIO clock */
  RCC_APB2PeriphClockCmd(LCD_E_GPIO_CLK | LCD_RW_GPIO_CLK | LCD_RS_GPIO_CLK | LCD_D_GPIO_CLK, ENABLE);

  /* Configure E in Output Push-Pull mode */
  GPIO_InitStructure.GPIO_Pin = LCD_E_PIN;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_Init(LCD_E_GPIO_PORT, &GPIO_InitStructure);
  
  /* Configure RW, RS in Output Push-Pull mode */
  GPIO_InitStructure.GPIO_Pin = LCD_RS_PIN;
  GPIO_Init(LCD_RS_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = LCD_RW_PIN;
  GPIO_Init(LCD_RW_GPIO_PORT, &GPIO_InitStructure);
  
  /* data port */
  GPIO_InitStructure.GPIO_Pin = LCD_D0_PIN;
  GPIO_Init(LCD_D_GPIO_PORT, &GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = LCD_D1_PIN;
  GPIO_Init(LCD_D_GPIO_PORT, &GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = LCD_D2_PIN;
  GPIO_Init(LCD_D_GPIO_PORT, &GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = LCD_D3_PIN;
  GPIO_Init(LCD_D_GPIO_PORT, &GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = LCD_D4_PIN;
  GPIO_Init(LCD_D_GPIO_PORT, &GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = LCD_D5_PIN;
  GPIO_Init(LCD_D_GPIO_PORT, &GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = LCD_D6_PIN;
  GPIO_Init(LCD_D_GPIO_PORT, &GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = LCD_D7_PIN;
  GPIO_Init(LCD_D_GPIO_PORT, &GPIO_InitStructure);
  
  GPIO_WriteBit(LCD_E_GPIO_PORT, LCD_E_PIN, Bit_RESET);
  GPIO_WriteBit(LCD_RW_GPIO_PORT, LCD_RW_PIN, Bit_RESET);
  GPIO_WriteBit(LCD_RS_GPIO_PORT, LCD_RS_PIN, Bit_RESET);
}

void lcd_init(void)
{
  LCD_CtrlLinesConfig();
  delay_100us(151);
  LCD_write(0x38, 0);
  delay_100us(1);
  LCD_write(0x0c, 0);
  delay_100us(1);
  LCD_write(0x06, 0);
  delay_100us(1);
  LCD_write(0x01, 0);
  delay_100us(16);
  
  lcd_clear();
}

static void LCD_write(uint8_t data, uint8_t rs)
{
  uint8_t mask = 0x01;
  if(rs)
    GPIO_WriteBit(LCD_RS_GPIO_PORT, LCD_RS_PIN, Bit_SET);
  else
    GPIO_WriteBit(LCD_RS_GPIO_PORT, LCD_RS_PIN, Bit_RESET);
  
  if(data & mask)
    GPIO_WriteBit(LCD_D_GPIO_PORT, LCD_D0_PIN, Bit_SET);
  else
    GPIO_WriteBit(LCD_D_GPIO_PORT, LCD_D0_PIN, Bit_RESET);
  mask = mask << 1;
  if(data & mask)
    GPIO_WriteBit(LCD_D_GPIO_PORT, LCD_D1_PIN, Bit_SET);
  else
    GPIO_WriteBit(LCD_D_GPIO_PORT, LCD_D1_PIN, Bit_RESET);
  mask = mask << 1;
  if(data & mask)
    GPIO_WriteBit(LCD_D_GPIO_PORT, LCD_D2_PIN, Bit_SET);
  else
    GPIO_WriteBit(LCD_D_GPIO_PORT, LCD_D2_PIN, Bit_RESET);
  mask = mask << 1;
  if(data & mask)
    GPIO_WriteBit(LCD_D_GPIO_PORT, LCD_D3_PIN, Bit_SET);
  else
    GPIO_WriteBit(LCD_D_GPIO_PORT, LCD_D3_PIN, Bit_RESET);
  mask = mask << 1;
  if(data & mask)
    GPIO_WriteBit(LCD_D_GPIO_PORT, LCD_D4_PIN, Bit_SET);
  else
    GPIO_WriteBit(LCD_D_GPIO_PORT, LCD_D4_PIN, Bit_RESET);
  mask = mask << 1;
  if(data & mask)
    GPIO_WriteBit(LCD_D_GPIO_PORT, LCD_D5_PIN, Bit_SET);
  else
    GPIO_WriteBit(LCD_D_GPIO_PORT, LCD_D5_PIN, Bit_RESET);
  mask = mask << 1;
  if(data & mask)
    GPIO_WriteBit(LCD_D_GPIO_PORT, LCD_D6_PIN, Bit_SET);
  else
    GPIO_WriteBit(LCD_D_GPIO_PORT, LCD_D6_PIN, Bit_RESET);
  mask = mask << 1;
  if(data & mask)
    GPIO_WriteBit(LCD_D_GPIO_PORT, LCD_D7_PIN, Bit_SET);
  else
    GPIO_WriteBit(LCD_D_GPIO_PORT, LCD_D7_PIN, Bit_RESET);
  
  GPIO_WriteBit(LCD_E_GPIO_PORT, LCD_E_PIN, Bit_SET);
  GPIO_WriteBit(LCD_E_GPIO_PORT, LCD_E_PIN, Bit_RESET);
}

void lcd_write_str(int8_t* str, uint8_t x, uint8_t y)
{
  LCD_write(0x80 | ((y * 0xc0) + x), 0);
  delay_100us(1);
  while(*str != 0)
  {
    LCD_write(*str++, 1);
    delay_100us(1);
  }
}

void lcd_clear(void)
{
  LCD_write(0x01, 0);
  delay_100us(16);
}

void lcd_curon(void)
{
  LCD_write(0x0e, 0);
  delay_100us(1);
}
  
void lcd_curoff(void)
{
  LCD_write(0x0c, 0);
  delay_100us(1);
}

void lcd_curleft(void)
{
  LCD_write(0x10, 0);
  delay_100us(1);
}

void lcd_curright(void)
{
  LCD_write(0x14, 0);
  delay_100us(1);
}


void lcd_bl_init(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
  TIM_OCInitTypeDef  TIM_OCInitStructure;
  uint16_t CCR1_Val = 1999;
  uint16_t CCR2_Val = 249;
  uint16_t CCR3_Val = 166;
  uint16_t CCR4_Val = 83;
  uint16_t PrescalerValue = 0;
  
  /* TIM4 clock enable */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, ENABLE);

  /* GPIOA and GPIOB clock enable */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB | RCC_APB2Periph_AFIO, ENABLE);
  
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  GPIO_Init(GPIOB, &GPIO_InitStructure);
  
  /* Compute the prescaler value */
  PrescalerValue = (uint16_t) (SystemCoreClock / 24000000) - 1;
  /* Time base configuration */
  TIM_TimeBaseStructure.TIM_Period = 1999;
  TIM_TimeBaseStructure.TIM_Prescaler = PrescalerValue;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

  TIM_TimeBaseInit(TIM4, &TIM_TimeBaseStructure);

  /* PWM1 Mode configuration: Channel3 */
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = CCR1_Val;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

  TIM_OC3Init(TIM4, &TIM_OCInitStructure);

  TIM_OC3PreloadConfig(TIM4, TIM_OCPreload_Enable);

  TIM_ARRPreloadConfig(TIM4, ENABLE);

  /* TIM4 enable counter */
  TIM_Cmd(TIM4, ENABLE);
}

// value range : 0~20
void lcd_bl_set(uint16_t value)
{
  TIM4->CCR3 = value * 100;
}

static void delay(vu32 nCount)
{
  vu32 index = 0; 
  for(index = (2400 * nCount); index != 0; index--)
  {
  }
}
